@@ -20,9 +20,9 @@
 //! This library is the official client for the free and open-source NoSQL database
 //! [Skytable](https://github.com/skytable/skytable). First, go ahead and install Skytable by
 //! following the instructions [here](https://docs.skytable.io/getting-started). This library supports
-//! all Skytable versions that work with the [Skyhash 1.0 Protocol](https://docs.skytable.io/protocol/skyhash).
+//! all Skytable versions that work with the [Skyhash 1.1 Protocol](https://docs.skytable.io/protocol/skyhash).
 //! This version of the library was tested with the latest Skytable release
-//! (release [0.6](https://github.com/skytable/skytable/releases/v0.6.0)).
+//! (release [0.7](https://github.com/skytable/skytable/releases/v0.7.0)).
 //!
 //! ## Using this library
 //!
@@ -38,12 +38,12 @@
 //!
 //! First add this to your `Cargo.toml` file:
 //! ```toml
-//! skytable = "0.4.0"
+//! skytable = "0.5.0"
 //! ```
 //! Now open up your `src/main.rs` file and establish a connection to the server while also adding some
 //! imports:
 //! ```no_run
-//! use skytable::{Connection, Query, Response, Element};
+//! use skytable::{Connection, Query, Element};
 //! fn main() -> std::io::Result<()> {
 //!     let mut con = Connection::new("127.0.0.1", 2003)?;
 //!     Ok(())
@@ -52,29 +52,49 @@
 //!
 //! Now let's run a [`Query`]! Change the previous code block to:
 //! ```no_run
-//! use skytable::{Connection, Query, Response, Element};
-//! fn main() -> std::io::Result<()> {
+//! use skytable::{error, Connection, Query, Element};
+//! fn main() -> Result<(), error::Error> {
 //!     let mut con = Connection::new("127.0.0.1", 2003)?;
 //!     let query = Query::from("heya");
 //!     let res = con.run_simple_query(&query)?;
-//!     assert_eq!(res, Response::Item(Element::String("HEY!".to_owned())));
+//!     assert_eq!(res, Element::String("HEY".to_owned()));
 //!     Ok(())
 //! }
 //! ```
-//!
 //! Way to go &mdash; you're all set! Now go ahead and run more advanced queries!
 //!
+//! ## Binary data
+//!
+//! The `default:default` keyspace has the following declaration:
+//! ```json
+//! Keymap { data:(binstr,binstr), volatile:false }
+//! ```
+//!
+//! This means that the default keyspace is ready to store binary data. Let's say
+//! you wanted to `SET` the value of a key called `bindata` to some binary data stored
+//! in a `Vec<u8>`. You can achieve this with the [`RawString`](types::RawString) type:
+//!
+//! ```no_run
+//! use skytable::actions::Actions;
+//! use skytable::sync::Connection;
+//! use skytable::types::RawString;
+//!
+//! let mut con = Connection::new("127.0.0.1", 2003).unwrap();
+//! let mybinarydata = RawString::from(vec![1, 2, 3, 4]);
+//! assert!(con.set("bindata", mybinarydata).unwrap());
+//! ```
+//!
 //! ## Going advanced
 //!
 //! Now that you know how you can run basic queries, check out the [`actions`] module documentation for learning
 //! to use actions and the [`types`] module documentation for implementing your own Skyhash serializable
-//! types.
+//! types. You can also find the [latest examples here](https://github.com/skytable/client-rust/tree/next/examples)
 //!
 //! ## Async API
 //!
 //! If you need to use an `async` API, just change your import to:
 //! ```toml
-//! skytable = { version = "0.4.0", features=["async"], default-features=false }
+//! skytable = { version = "0.5.0", features=["async"], default-features=false }
 //! ```
 //! You can now establish a connection by using `skytable::AsyncConnection::new()`, adding `.await`s wherever
 //! necessary. Do note that you'll the [Tokio runtime](https://tokio.rs).
@@ -84,7 +104,7 @@
 //! With this client driver, it is possible to use both sync and `async` APIs **at the same time**. To do
 //! this, simply change your import to:
 //! ```toml
-//! skytable = { version="0.4.0", features=["sync", "async"] }
+//! skytable = { version="0.5.0", features=["sync", "async"] }
 //! ```
 //!
 //! ## TLS
@@ -93,15 +113,15 @@
 //!
 //! ### Using TLS with sync interfaces
 //! ```toml
-//! skytable = { version="0.4.0", features=["sync","ssl"] }
+//! skytable = { version="0.5.0", features=["sync","ssl"] }
 //! ```
-//! You can now use the async `sync::TlsConnection` object.
+//! You can now use the async [TlsConnection](`sync::TlsConnection`) object.
 //!
 //! ### Using TLS with async interfaces
 //! ```toml
-//! skytable = { version="0.4.0", features=["async","aio-ssl"], default-features=false }
+//! skytable = { version="0.5.0", features=["async","aio-ssl"], default-features=false }
 //! ```
-//! You can now use the async `aio::TlsConnection` object.
+//! You can now use the async [TlsConnection](`aio::TlsConnection`) object.
 //!
 //! ### _Packed TLS_ setup
 //!
@@ -126,6 +146,7 @@
 mod util;
 pub mod actions;
 mod deserializer;
+pub mod error;
 mod respcode;
 pub mod types;
 use crate::types::GetIterator;
@@ -145,6 +166,114 @@ cfg_sync!(
     pub use sync::Connection;
 );
 
+/// A generic result type
+pub type SkyRawResult<T> = Result<T, self::error::Error>;
+/// A specialized error type for queries
+pub type SkyResult = SkyRawResult<Element>;
+
+#[derive(Debug)]
+/// A connection builder for easily building connections
+///
+/// ## Example (sync)
+/// ```no_run
+/// use skytable::ConnectionBuilder;
+/// let con =
+///     ConnectionBuilder::new()
+///     .set_host("127.0.0.1")
+///     .set_port(2003)
+///     .get_connection()
+///     .unwrap();
+/// ```
+///
+/// ## Example (async)
+/// ```no_test
+/// use skytable::ConnectionBuilder;
+/// async fn main() {
+///     let con =
+///         ConnectionBuilder::new()
+///         .set_host("127.0.0.1")
+///         .set_port(2003)
+///         .get_async_connection()
+///         .await
+///         .unwrap();
+/// }
+/// ```
+pub struct ConnectionBuilder<'a> {
+    port: Option<u16>,
+    host: Option<&'a str>,
+}
+
+impl<'a> Default for ConnectionBuilder<'a> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+impl<'a> ConnectionBuilder<'a> {
+    /// Create an empty connection builder
+    pub fn new() -> Self {
+        Self {
+            port: None,
+            host: None,
+        }
+    }
+    /// Set the port
+    pub fn set_port(mut self, port: u16) -> Self {
+        self.port = Some(port);
+        self
+    }
+    /// Set the host
+    pub fn set_host(mut self, host: &'a str) -> Self {
+        self.host = Some(host);
+        self
+    }
+    cfg_sync! {
+        /// Get a [sync connection](sync::Connection) to the database
+        pub fn get_connection(&self) -> SkyRawResult<sync::Connection> {
+            let con =
+                sync::Connection::new(self.host.unwrap_or("127.0.0.1"), self.port.unwrap_or(2003))?;
+            Ok(con)
+        }
+        cfg_sync_ssl_any! {
+            /// Get a [sync TLS connection](sync::TlsConnection) to the database
+            pub fn get_tls_connection(
+                &self,
+                sslcert: String,
+            ) -> SkyRawResult<sync::TlsConnection> {
+                let con = sync::TlsConnection::new(
+                    self.host.unwrap_or("127.0.0.1"),
+                    self.port.unwrap_or(2003),
+                    &sslcert,
+                )?;
+                Ok(con)
+            }
+        }
+    }
+    cfg_async! {
+        /// Get an [async connection](aio::Connection) to the database
+        pub async fn get_async_connection(&self) -> SkyRawResult<aio::Connection> {
+            let con = aio::Connection::new(self.host.unwrap_or("127.0.0.1"), self.port.unwrap_or(2003))
+                .await?;
+            Ok(con)
+        }
+        cfg_async_ssl_any! {
+            /// Get an [async TLS connection](aio::TlsConnection) to the database
+            pub async fn get_async_tls_connection(
+                &self,
+                sslcert: String,
+            ) -> SkyRawResult<aio::TlsConnection> {
+                let con = aio::TlsConnection::new(
+                    self.host.unwrap_or("127.0.0.1"),
+                    self.port.unwrap_or(2003),
+                    &sslcert,
+                )
+                .await?;
+                Ok(con)
+            }
+        }
+    }
+}
+
 #[macro_export]
 /// A macro that can be used to easily create queries with _almost_ variadic properties.
 /// Where you'd normally create queries like this:
@@ -159,7 +288,7 @@ cfg_sync!(
 /// ```
 macro_rules! query {
     ($($arg:expr),+) => {
-        skytable::Query::new()$(.arg($arg))*
+        $crate::Query::new()$(.arg($arg))*
     };
 }
 
@@ -182,6 +311,9 @@ macro_rules! query {
 /// let q2 = Query::from(vec!["mset", "x", "100", "y", "200", "z", "300"]);
 /// let q3 = Query::from("get").arg("x");
 /// ```
+/// **Important:** You should use the [`RawString`](types::RawString) type if you're willing to directly add things like
+/// `Vec<u8>` to your query.
+///
 /// Finally, queries can also be created by taking references. For example:
 /// ```
 /// use skytable::Query;
@@ -230,20 +362,18 @@ impl Query {
         arg.push_into_query(&mut self);
         self
     }
-    pub(in crate) fn _push_arg(&mut self, arg: impl IntoSkyhashBytes) {
-        let arg = arg.as_string();
+    pub(in crate) fn _push_arg(&mut self, arg: Vec<u8>) {
         if arg.is_empty() {
             panic!("Argument cannot be empty")
         }
         // A data element will look like:
-        // `+<bytes_in_next_line>\n<data>`
-        self.data.push(b'+');
+        // `<bytes_in_next_line>\n<data>`
         let bytes_in_next_line = arg.len().to_string().into_bytes();
         self.data.extend(bytes_in_next_line);
         // add the LF char
         self.data.push(b'\n');
         // Add the data itself, which is `arg`
-        self.data.extend(arg.into_bytes());
+        self.data.extend(arg);
         self.data.push(b'\n'); // add the LF char
         self.size_count += 1;
     }
@@ -267,15 +397,19 @@ impl Query {
         U: IntoSkyhashBytes,
     {
         v1.get_iter().zip(v2.get_iter()).for_each(|(a, b)| {
-            self.push(a.as_string());
-            self.push(b.as_string());
+            self._push_arg(a.to_bytes());
+            self._push_arg(b.to_bytes());
         });
         self
     }
-    /// Number of items in the datagroup
-    pub(crate) fn __len(&self) -> usize {
+    /// Returns the number of arguments in this query
+    pub fn len(&self) -> usize {
         self.size_count
     }
+    /// Check if the query is empty
+    pub fn is_empty(&self) -> bool {
+        self.size_count == 0
+    }
     fn get_holding_buffer(&self) -> &[u8] {
         &self.data
     }
@@ -288,8 +422,8 @@ impl Query {
             // Write the metaframe
             stream.write_all(b"*1\n").await?;
             // Add the dataframe
-            let number_of_items_in_datagroup = self.__len().to_string().into_bytes();
-            stream.write_all(&[b'_']).await?;
+            let number_of_items_in_datagroup = self.len().to_string().into_bytes();
+            stream.write_all(&[b'~']).await?;
             stream.write_all(&number_of_items_in_datagroup).await?;
             stream.write_all(&[b'\n']).await?;
             stream.write_all(self.get_holding_buffer()).await?;
@@ -306,8 +440,8 @@ impl Query {
             // Write the metaframe
             stream.write_all(b"*1\n")?;
             // Add the dataframe
-            let number_of_items_in_datagroup = self.__len().to_string().into_bytes();
-            stream.write_all(&[b'_'])?;
+            let number_of_items_in_datagroup = self.len().to_string().into_bytes();
+            stream.write_all(&[b'~'])?;
             stream.write_all(&number_of_items_in_datagroup)?;
             stream.write_all(&[b'\n'])?;
             stream.write_all(self.get_holding_buffer())?;
@@ -324,8 +458,8 @@ impl Query {
         /// return the raw query that would be written to the stream, serialized using the Skyhash serialization protocol
         pub fn into_raw_query(self) -> Vec<u8> {
             let mut v = Vec::with_capacity(self.data.len());
-            v.extend(b"*1\n_");
-            v.extend(self.__len().to_string().into_bytes());
+            v.extend(b"*1\n~");
+            v.extend(self.len().to_string().into_bytes());
             v.extend(b"\n");
             v.extend(self.get_holding_buffer());
             v
@@ -347,8 +481,6 @@ impl Query {
             // the newline
             len += 1;
             element_lengths.iter().for_each(|elem| {
-                // the tsymbol
-                len += 1;
                 // the digit length
                 len += dig_count(*elem);
                 // the newline
@@ -363,25 +495,6 @@ impl Query {
     );
 }
 
-/// # Responses
-///
-/// This enum represents responses returned by the server. This can either be an array (or bulk), a single item
-/// or can be a parse error if the server returned some data but it couldn't be parsed into the expected type
-/// or it can be an invalid response in the event the server sent some invalid data.
-/// This enum is `#[non_exhaustive]` as more types of responses can be added in the future.
-#[derive(Debug, PartialEq)]
-#[non_exhaustive]
-pub enum Response {
-    /// The server sent an invalid response
-    InvalidResponse,
-    /// The server responded with _something_. This can be any of the [`Element`] variants
-    Item(Element),
-    /// We failed to parse data
-    ParseError,
-    /// The server sent some data of a type that this client doesn't support
-    UnsupportedDataType,
-}
-
 cfg_dbg!(
     #[test]
     fn my_query() {
@@ -392,46 +505,3 @@ cfg_dbg!(
         assert_eq!(ma_query_len, q_len);
     }
 );
-
-pub mod error {
-    //! Errors
-    cfg_ssl_any!(
-        use std::fmt;
-        /// Errors that may occur while initiating an [async TLS connection](crate::aio::TlsConnection)
-        /// or a [sync TLS connection](crate::sync::TlsConnection)
-        #[derive(Debug)]
-        pub enum SslError {
-            /// An [I/O Error](std::io::Error) occurred
-            IoError(std::io::Error),
-            /// An [SSL Error](openssl::error::Error) occurred
-            SslError(openssl::ssl::Error),
-        }
-
-        impl From<openssl::ssl::Error> for SslError {
-            fn from(e: openssl::ssl::Error) -> Self {
-                Self::SslError(e)
-            }
-        }
-
-        impl From<std::io::Error> for SslError {
-            fn from(e: std::io::Error) -> Self {
-                Self::IoError(e)
-            }
-        }
-
-        impl From<openssl::error::ErrorStack> for SslError {
-            fn from(e: openssl::error::ErrorStack) -> Self {
-                Self::SslError(e.into())
-            }
-        }
-
-        impl fmt::Display for SslError {
-            fn fmt(&self, f: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
-                match self {
-                    Self::IoError(e) => write!(f, "{}", e),
-                    Self::SslError(e) => write!(f, "{}", e),
-                }
-            }
-        }
-    );
-}
